package dev.adriankuta.flights.ui.home.components

import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.itemsIndexed
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Search
import androidx.compose.material3.Card
import androidx.compose.material3.CardDefaults
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.tooling.preview.PreviewParameter
import androidx.compose.ui.unit.dp
import dev.adriankuta.flights.domain.types.Flight
import dev.adriankuta.flights.domain.types.Trip
import dev.adriankuta.flights.domain.types.TripDate
import dev.adriankuta.flights.domain.types.TripFlight
import dev.adriankuta.flights.ui.home.SearchResultUiState
import dev.adriankuta.flights.ui.home.extensions.hasFlights
import dev.adriankuta.flights.ui.home.sample.FlightPreviewParameterProvider

@Composable
internal fun SearchResults(
    uiState: SearchResultUiState,
    modifier: Modifier = Modifier,
) {
    LazyColumn(
        modifier = modifier.fillMaxWidth(),
    ) {
        stickyHeader {
            Text(
                text = "Search Results",
                style = MaterialTheme.typography.headlineSmall,
                fontWeight = FontWeight.Bold,
                modifier = Modifier.padding(bottom = 16.dp),
            )
        }

        when (uiState) {
            is SearchResultUiState.Error -> item { Text("Error") }
            SearchResultUiState.Idle -> Unit
            SearchResultUiState.Loading -> item { Text("Loading") }
            is SearchResultUiState.Success -> {
                if (uiState.flight.hasFlights()) {
                    itemsIndexed(uiState.flight.trips) { index, trip ->
                        TripCard(trip = trip, currency = uiState.flight.currency)
                        if (index < uiState.flight.trips.lastIndex) {
                            Spacer(modifier = Modifier.height(16.dp))
                        }
                    }
                } else {
                    item { EmptyListComponent() }
                }
            }
        }
    }
}

@Composable
private fun TripCard(
    trip: Trip,
    currency: String,
    modifier: Modifier = Modifier,
) {
    Card(
        modifier = modifier.fillMaxWidth(),
        elevation = CardDefaults.cardElevation(defaultElevation = 4.dp),
    ) {
        Column(
            modifier = Modifier.padding(16.dp),
        ) {
            Text(
                text = "${trip.origin} → ${trip.destination}",
                style = MaterialTheme.typography.titleMedium,
                fontWeight = FontWeight.Bold,
            )

            Spacer(modifier = Modifier.height(8.dp))

            Column {
                trip.dates.forEachIndexed { index, tripDate ->
                    TripDateItem(tripDate = tripDate, currency = currency)
                    if (index != trip.dates.lastIndex) {
                        HorizontalDivider(modifier = Modifier.padding(vertical = 8.dp))
                    }
                }
            }
        }
    }
}

@Composable
private fun TripDateItem(
    tripDate: TripDate,
    currency: String,
    modifier: Modifier = Modifier,
) {
    Column(modifier = modifier.fillMaxWidth()) {
        Text(
            text = "Date: ${tripDate.dateOut}",
            style = MaterialTheme.typography.bodyMedium,
            fontWeight = FontWeight.Medium,
        )

        Spacer(modifier = Modifier.height(8.dp))

        tripDate.flights.forEach { flight ->
            FlightItem(flight = flight, currency = currency)
            Spacer(modifier = Modifier.height(8.dp))
        }
    }
}

@Composable
private fun FlightItem(
    flight: TripFlight,
    currency: String,
    modifier: Modifier = Modifier,
) {
    Card(
        modifier = modifier.fillMaxWidth(),
        colors = CardDefaults.cardColors(
            containerColor = MaterialTheme.colorScheme.surfaceVariant,
        ),
    ) {
        Column(
            modifier = Modifier.padding(12.dp),
        ) {
            Row(
                verticalAlignment = Alignment.CenterVertically,
            ) {
                Text(
                    text = "Flight: ${flight.flightNumber}",
                    style = MaterialTheme.typography.bodyMedium,
                    modifier = Modifier.weight(1f),
                )
                Text(
                    text = "Duration: ${flight.duration}",
                    style = MaterialTheme.typography.bodyMedium,
                )
            }

            Spacer(modifier = Modifier.height(4.dp))

            Text(
                text = "Operated by: ${flight.operatedBy}",
                style = MaterialTheme.typography.bodySmall,
            )

            Spacer(modifier = Modifier.height(4.dp))

            // Display price information
            val totalPrice = flight.regularFare.fares.sumOf { it.amount * it.count }
            Text(
                text = "Price: $totalPrice $currency",
                style = MaterialTheme.typography.bodyMedium,
                fontWeight = FontWeight.Bold,
            )

            Text(
                text = "Seats available: ${flight.faresLeft}",
                style = MaterialTheme.typography.bodySmall,
            )
        }
    }
}

@Composable
private fun EmptyListComponent(
    modifier: Modifier = Modifier,
) {
    Box(
        modifier = modifier
            .fillMaxSize()
            .padding(16.dp),
        contentAlignment = Alignment.Center,
    ) {
        Column(
            horizontalAlignment = Alignment.CenterHorizontally,
        ) {
            Icon(
                imageVector = Icons.Default.Search,
                contentDescription = null,
                modifier = Modifier.padding(bottom = 16.dp),
                tint = MaterialTheme.colorScheme.primary,
            )
            Text(
                text = "No flights found",
                style = MaterialTheme.typography.titleMedium,
                fontWeight = FontWeight.Bold,
                textAlign = TextAlign.Center,
            )
            Spacer(modifier = Modifier.height(8.dp))
            Text(
                text = "Try changing your search criteria",
                style = MaterialTheme.typography.bodyMedium,
                textAlign = TextAlign.Center,
            )
        }
    }
}

@Preview(showBackground = true)
@Composable
private fun EmptyListPreview() {
    EmptyListComponent()
}

@Preview(showBackground = true)
@Composable
private fun SearchResultsPreview(@PreviewParameter(FlightPreviewParameterProvider::class) flight: Flight) {
    SearchResults(uiState = SearchResultUiState.Success(flight))
}

@Preview(showBackground = true)
@Composable
private fun SearchResultsEmptyPreview() {
    SearchResults(
        uiState = SearchResultUiState.Success(
            Flight(
                currency = "EUR",
                currPrecision = 2,
                trips = emptyList(),
            ),
        ),
    )
}
